<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Model;

use Magento\Framework\Serialize\SerializerInterface;
use MageWorx\XmlSitemap\Model\WriterInterface;
use MageWorx\XmlSitemap\Model\Writer as MageworxWriter;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\Store as StoreModel;
use function strpos;

/**
 * Class Writer
 * @package FiloBlu\Refilo\Model
 */
class Writer implements WriterInterface
{

    /** @var string  */
    public const XML_REFILO_MARKETING_XML_SITEMAP_IMAGE_RESOLUTIONS = 'mageworx_seo/xml_sitemap/image_resolutions';

    /**
     * @var string
     */
    public $storeBaseUrl;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var MageworxWriter
     */
    private $writer;
    /**
     * @var SerializerInterface
     */
    private $serializer;

    /**
     * @param MageworxWriter $writer
     * @param ScopeConfigInterface $scopeConfig
     * @param SerializerInterface $serializer
     */
    public function __construct(
        MageworxWriter       $writer,
        ScopeConfigInterface $scopeConfig,
        SerializerInterface  $serializer
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->writer = $writer;
        $this->serializer = $serializer;
    }

    /**
     * @param $rawUrl
     * @param $lastmod
     * @param $changefreq
     * @param $priority
     * @param $imageUrls
     * @param $videoUrls
     * @return void
     * @throws LocalizedException
     */
    public function write($rawUrl, $lastmod, $changefreq, $priority, $imageUrls = false, $videoUrls = false)
    {
        if ($imageUrls && is_countable($imageUrls->getCollection()) && count($imageUrls->getCollection())) {
            $imageUrls = $this->updateImageUrls($imageUrls);
        }

        $this->writer->write(
            $rawUrl,
            $lastmod,
            $changefreq,
            $priority,
            $imageUrls,
            $videoUrls
        );
    }

    /**
     * @param mixed $imageUrls
     * @return mixed
     */
    public function updateImageUrls($imageUrls)
    {
        $resolutions = $this->getImageResolutionsFromConfig();

        if (!$resolutions) {
            return $imageUrls;
        }

        $collection = $imageUrls->getCollection() ?? [];

        $urlObjectArray = [];
        foreach ($collection as $image) {
            $url = $image->getUrl();

            if (!$url) {
                continue;
            }

            $position = strpos($url, 'media/catalog/product');
            if ($position === false) {
                continue;
            }

            $parsedUrl = parse_url($url);

            if(!is_array($parsedUrl)){
                continue;
            }

            $paths = explode('/', $parsedUrl['path']);

            if (!is_array($paths) || count($paths) < 3) {
                continue;
            }

            $imageName = array_pop($paths);
            if (empty($imageName)) {
                continue;
            }

            $spread2 = array_pop($paths);
            if (empty($spread2)) {
                continue;
            }

            $spread1 = array_pop($paths);
            if (empty($spread1)) {
                continue;
            }

            foreach ($resolutions as $resolution) {
                $newUrl = sprintf(
                    '%s://%s/rx/%sx,ofmt_webp/media/catalog/product/%s/%s/%s',
                    $parsedUrl['scheme'],
                    $parsedUrl['host'],
                    $resolution,
                    $spread1,
                    $spread2,
                    $imageName
                );

                $urlObject = clone $image;
                $urlObject->setData('url', $newUrl);

                $urlObjectArray[] = $urlObject;
            }
        }

        $imageUrls->setCollection($urlObjectArray);

        return $imageUrls;
    }

    /**
     * @return array
     */
    public function getImageResolutionsFromConfig(): array
    {
        $serializedResolutions = $this->scopeConfig->getValue(
            self::XML_REFILO_MARKETING_XML_SITEMAP_IMAGE_RESOLUTIONS
        );

        if (empty($serializedResolutions)) {
            return [];
        }

        $resolutionsArray = [];
        $data = $this->serializer->unserialize($serializedResolutions);
        foreach ($data as $resolution) {
            $resolutionsArray[] = $resolution['resolution'];
        }

        return $resolutionsArray;
    }

    /**
     * @param $filePath
     * @param $fileDir
     * @param $fileName
     * @param $tempFilePath
     * @param $storeBaseUrl
     * @param $storeId
     * @param $serverPath
     * @return void
     */
    public function init(
        $filePath,
        $fileDir,
        $fileName,
        $tempFilePath,
        $storeBaseUrl = false,
        $storeId = StoreModel::DEFAULT_STORE_ID,
        $serverPath = ''
    )
    {
        call_user_func_array( [$this->writer, 'init'] , func_get_args());
        $this->storeBaseUrl = $storeBaseUrl;
    }

    /**
     * @return void
     */
    public function startWriteXml()
    {
        $this->writer->startWriteXml();
    }

    /**
     * @return void
     * @throws LocalizedException
     */
    public function endWriteXml()
    {
        $this->writer->endWriteXml();
    }

    /**
     * @return int
     */
    public function getWritedImages()
    {
        return $this->writer->getWritedImages();
    }

    /**
     * @param array $alternateUrl
     * @return void
     */
    public function setAlternateUrls(array $alternateUrl)
    {
        $this->writer->setAlternateUrls($alternateUrl);
    }
}
